/*  NDX.H   . . .   Header file for index package   */

#ifndef	NDX_H
#define NDX_H

#define STACKDEPTH  10      /* Maximum tree height */
#define NODESIZE    1024    /* Bytes per node */
#define SEEKADR     unsigned long
#define MAXKEY      128     /* Maximum key length */
#define NODUPS      0       /* No duplicate keys */
#define FIFO        1       /* ISAM, first-in-first-out */
#define LIFO        2       /* ISAM, last-in-first-out */

typedef struct{
    SEEKADR     ptr;        /* Pointer to lower level node (lesser keys) */
    SEEKADR     seekadr;     /* Record number or offset in data file */
    char        key[MAXKEY];   /* The ASCIZ key */
} KEY;

/*  Note that each node can have a final pointer to the next lower level which
    is not counted in end_keys, but is at that relative position.
*/

typedef struct{        /* Index file header */
    SEEKADR    root;       /* File offset of root node */
	SEEKADR	   next;	   /* Offset of end of file */
	SEEKADR    freelist;   /* Chain of unused nodes */
	unsigned   char minor, major; /* Major and minor version */
    unsigned   mode;       /* Retrieval mode (FIFO, etc.) */
    unsigned   nodesize;
} HDR;

typedef struct {        /* Stack frame */
    SEEKADR  node;          /* Offset of node in index file */
    unsigned off;           /* Offset of key in node->keys */
} STACK;

/* The stack holds the current retrieval state of the index:

*/

typedef struct{         /* In memory index structure */
	int (* cmpfn)();		/* Comparison function */
	KEY			key;		/* Current key */
	HDR			hdr;		/* Index file header */
	int         level;      /* Stack pointer */
	STACK		stack[STACKDEPTH];
	int         handle;     /* File handle */
    char        filename[80];   /* Filename of index (not data) file */
} INDEX;

/*	Function prototypes */

INDEX *xcreate(char * indexname, int (*compfn)(), unsigned mode);
INDEX *xopen  (char * indexname, int (*compfn)());
void xclose(INDEX * index);
SEEKADR xinsert(INDEX * index, char * key, SEEKADR seekadr);
xdelete(INDEX * index, char * key, SEEKADR seekadr);
SEEKADR xfind(INDEX * index, char * key);
SEEKADR xfirst(INDEX * index, unsigned last);
SEEKADR xnext(INDEX * index);
SEEKADR xprev(INDEX * index);

#endif	/* NDX_H */

/*  End of NDX.H    */

